@@ -9,6 +9,7 @@ such as `.PathPatch` and `.PathCollection`, can be used for convenient `Path`
 visualisation.
 """
 
+import copy
 from functools import lru_cache
 from weakref import WeakValueDictionary
 
@@ -27,38 +28,38 @@ class Path:
 
     The underlying storage is made up of two parallel numpy arrays:
 
-    - *vertices*: an Nx2 float array of vertices
-    - *codes*: an N-length uint8 array of vertex types, or None
+    - *vertices*: an (N, 2) float array of vertices
+    - *codes*: an N-length `numpy.uint8` array of path codes, or None
 
     These two arrays always have the same length in the first
     dimension.  For example, to represent a cubic curve, you must
-    provide three vertices as well as three codes ``CURVE3``.
+    provide three vertices and three `CURVE4` codes.
 
     The code types are:
 
-    - ``STOP``   :  1 vertex (ignored)
+    - `STOP`   :  1 vertex (ignored)
         A marker for the end of the entire path (currently not required and
         ignored)
 
-    - ``MOVETO`` :  1 vertex
+    - `MOVETO` :  1 vertex
         Pick up the pen and move to the given vertex.
 
-    - ``LINETO`` :  1 vertex
+    - `LINETO` :  1 vertex
         Draw a line from the current position to the given vertex.
 
-    - ``CURVE3`` :  1 control point, 1 endpoint
-        Draw a quadratic Bezier curve from the current position, with the given
+    - `CURVE3` :  1 control point, 1 endpoint
+        Draw a quadratic Bézier curve from the current position, with the given
         control point, to the given end point.
 
-    - ``CURVE4`` :  2 control points, 1 endpoint
-        Draw a cubic Bezier curve from the current position, with the given
+    - `CURVE4` :  2 control points, 1 endpoint
+        Draw a cubic Bézier curve from the current position, with the given
         control points, to the given end point.
 
-    - ``CLOSEPOLY`` : 1 vertex (ignored)
+    - `CLOSEPOLY` : 1 vertex (ignored)
         Draw a line segment to the start point of the current polyline.
 
-    If *codes* is None, it is interpreted as a ``MOVETO`` followed by a series
-    of ``LINETO``.
+    If *codes* is None, it is interpreted as a `MOVETO` followed by a series
+    of `LINETO`.
 
     Users of Path objects should not access the vertices and codes arrays
     directly.  Instead, they should use `iter_segments` or `cleaned` to get the
@@ -108,7 +109,7 @@ class Path:
             handled correctly by the Agg PathIterator and other consumers of
             path data, such as :meth:`iter_segments`.
         codes : array-like or None, optional
-            n-length array integers representing the codes of the path.
+            N-length array of integers representing the codes of the path.
             If not None, codes must be the same length as vertices.
             If None, *vertices* will be treated as a series of line segments.
         _interpolation_steps : int, optional
@@ -120,7 +121,7 @@ class Path:
             If *codes* is None and closed is True, vertices will be treated as
             line segments of a closed polygon.  Note that the last vertex will
             then be ignored (as the corresponding code will be set to
-            CLOSEPOLY).
+            `CLOSEPOLY`).
         readonly : bool, optional
             Makes the path behave in an immutable way and sets the vertices
             and codes as read-only arrays.
@@ -161,12 +162,12 @@ class Path:
     @classmethod
     def _fast_from_codes_and_verts(cls, verts, codes, internals_from=None):
         """
-        Creates a Path instance without the expense of calling the constructor.
+        Create a Path instance without the expense of calling the constructor.
 
         Parameters
         ----------
-        verts : numpy array
-        codes : numpy array
+        verts : array-like
+        codes : array
         internals_from : Path or None
             If not None, another `Path` from which the attributes
             ``should_simplify``, ``simplify_threshold``, and
@@ -187,6 +188,17 @@ class Path:
             pth._interpolation_steps = 1
         return pth
 
+    @classmethod
+    def _create_closed(cls, vertices):
+        """
+        Create a closed polygonal path going through *vertices*.
+
+        Unlike ``Path(..., closed=True)``, *vertices* should **not** end with
+        an entry for the CLOSEPATH; this entry is added by `._create_closed`.
+        """
+        v = _to_unmasked_float_array(vertices)
+        return cls(np.concatenate([v, v[:1]]), closed=True)
+
     def _update_values(self):
         self._simplify_threshold = mpl.rcParams['path.simplify_threshold']
         self._should_simplify = (
@@ -198,9 +210,7 @@ class Path:
 
     @property
     def vertices(self):
-        """
-        The list of vertices in the `Path` as an Nx2 numpy array.
-        """
+        """The vertices of the `Path` as an (N, 2) array."""
         return self._vertices
 
     @vertices.setter
@@ -213,12 +223,12 @@ class Path:
     @property
     def codes(self):
         """
-        The list of codes in the `Path` as a 1D numpy array.  Each
-        code is one of `STOP`, `MOVETO`, `LINETO`, `CURVE3`, `CURVE4`
-        or `CLOSEPOLY`.  For codes that correspond to more than one
-        vertex (`CURVE3` and `CURVE4`), that code will be repeated so
-        that the length of `self.vertices` and `self.codes` is always
-        the same.
+        The list of codes in the `Path` as a 1D array.
+
+        Each code is one of `STOP`, `MOVETO`, `LINETO`, `CURVE3`, `CURVE4` or
+        `CLOSEPOLY`.  For codes that correspond to more than one vertex
+        (`CURVE3` and `CURVE4`), that code will be repeated so that the length
+        of `vertices` and `codes` is always the same.
         """
         return self._codes
 
@@ -259,43 +269,37 @@ class Path:
         """
         return self._readonly
 
-    def __copy__(self):
+    def copy(self):
         """
         Return a shallow copy of the `Path`, which will share the
         vertices and codes with the source `Path`.
         """
-        import copy
         return copy.copy(self)
 
-    copy = __copy__
-
     def __deepcopy__(self, memo=None):
         """
         Return a deepcopy of the `Path`.  The `Path` will not be
         readonly, even if the source `Path` is.
         """
-        try:
-            codes = self.codes.copy()
-        except AttributeError:
-            codes = None
-        return self.__class__(
-            self.vertices.copy(), codes,
-            _interpolation_steps=self._interpolation_steps)
+        # Deepcopying arrays (vertices, codes) strips the writeable=False flag.
+        p = copy.deepcopy(super(), memo)
+        p._readonly = False
+        return p
 
     deepcopy = __deepcopy__
 
     @classmethod
     def make_compound_path_from_polys(cls, XY):
         """
-        Make a compound path object to draw a number
-        of polygons with equal numbers of sides XY is a (numpolys x
-        numsides x 2) numpy array of vertices.  Return object is a
-        :class:`Path`
+        Make a compound `Path` object to draw a number of polygons with equal
+        numbers of sides.
 
         .. plot:: gallery/misc/histogram_path.py
 
+        Parameters
+        ----------
+        XY : (numpolys, numsides, 2) array
         """
-
         # for each poly: 1 for the MOVETO, (numsides-1) for the LINETO, 1 for
         # the CLOSEPOLY; the vert for the closepoly is ignored but we still
         # need it to keep the codes aligned with the vertices
@@ -310,37 +314,32 @@ class Path:
         codes[numsides::stride] = cls.CLOSEPOLY
         for i in range(numsides):
             verts[i::stride] = XY[:, i]
-
         return cls(verts, codes)
 
     @classmethod
     def make_compound_path(cls, *args):
+        r"""
+        Concatenate a list of `Path`\s into a single `Path`, removing all `STOP`\s.
         """
-        Make a compound path from a list of Path objects. Blindly removes all
-        Path.STOP control points.
-        """
-        # Handle an empty list in args (i.e. no args).
         if not args:
             return Path(np.empty([0, 2], dtype=np.float32))
-        vertices = np.concatenate([x.vertices for x in args])
+        vertices = np.concatenate([path.vertices for path in args])
         codes = np.empty(len(vertices), dtype=cls.code_type)
         i = 0
         for path in args:
+            size = len(path.vertices)
             if path.codes is None:
-                codes[i] = cls.MOVETO
-                codes[i + 1:i + len(path.vertices)] = cls.LINETO
+                if size:
+                    codes[i] = cls.MOVETO
+                    codes[i+1:i+size] = cls.LINETO
             else:
-                codes[i:i + len(path.codes)] = path.codes
-            i += len(path.vertices)
-        # remove STOP's, since internal STOPs are a bug
-        not_stop_mask = codes != cls.STOP
-        vertices = vertices[not_stop_mask, :]
-        codes = codes[not_stop_mask]
-
-        return cls(vertices, codes)
+                codes[i:i+size] = path.codes
+            i += size
+        not_stop_mask = codes != cls.STOP  # Remove STOPs, as internal STOPs are a bug.
+        return cls(vertices[not_stop_mask], codes[not_stop_mask])
 
     def __repr__(self):
-        return "Path(%r, %r)" % (self.vertices, self.codes)
+        return f"Path({self.vertices!r}, {self.codes!r})"
 
     def __len__(self):
         return len(self.vertices)
@@ -413,7 +412,7 @@ class Path:
 
     def iter_bezier(self, **kwargs):
         """
-        Iterate over each bezier curve (lines included) in a Path.
+        Iterate over each Bézier curve (lines included) in a `Path`.
 
         Parameters
         ----------
@@ -422,15 +421,15 @@ class Path:
 
         Yields
         ------
-        B : matplotlib.bezier.BezierSegment
-            The bezier curves that make up the current path. Note in particular
-            that freestanding points are bezier curves of order 0, and lines
-            are bezier curves of order 1 (with two control points).
-        code : Path.code_type
+        B : `~matplotlib.bezier.BezierSegment`
+            The Bézier curves that make up the current path. Note in particular
+            that freestanding points are Bézier curves of order 0, and lines
+            are Bézier curves of order 1 (with two control points).
+        code : `~matplotlib.path.Path.code_type`
             The code describing what kind of curve is being returned.
-            Path.MOVETO, Path.LINETO, Path.CURVE3, Path.CURVE4 correspond to
-            bezier curves with 1, 2, 3, and 4 control points (respectively).
-            Path.CLOSEPOLY is a Path.LINETO with the control points correctly
+            `MOVETO`, `LINETO`, `CURVE3`, and `CURVE4` correspond to
+            Bézier curves with 1, 2, 3, and 4 control points (respectively).
+            `CLOSEPOLY` is a `LINETO` with the control points correctly
             chosen based on the start/end points of the current stroke.
         """
         first_vert = None
@@ -455,15 +454,24 @@ class Path:
             elif code == Path.STOP:
                 return
             else:
-                raise ValueError("Invalid Path.code_type: " + str(code))
+                raise ValueError(f"Invalid Path.code_type: {code}")
             prev_vert = verts[-2:]
 
-    @_api.delete_parameter("3.3", "quantize")
+    def _iter_connected_components(self):
+        """Return subpaths split at MOVETOs."""
+        if self.codes is None:
+            yield self
+        else:
+            idxs = np.append((self.codes == Path.MOVETO).nonzero()[0], len(self.codes))
+            for sl in map(slice, idxs, idxs[1:]):
+                yield Path._fast_from_codes_and_verts(
+                    self.vertices[sl], self.codes[sl], self)
+
     def cleaned(self, transform=None, remove_nans=False, clip=None,
-                quantize=False, simplify=False, curves=False,
+                *, simplify=False, curves=False,
                 stroke_width=1.0, snap=False, sketch=None):
         """
-        Return a new Path with vertices and codes cleaned according to the
+        Return a new `Path` with vertices and codes cleaned according to the
         parameters.
 
         See Also
@@ -496,19 +504,19 @@ class Path:
         Return whether the area enclosed by the path contains the given point.
 
         The path is always treated as closed; i.e. if the last code is not
-        CLOSEPOLY an implicit segment connecting the last vertex to the first
+        `CLOSEPOLY` an implicit segment connecting the last vertex to the first
         vertex is assumed.
 
         Parameters
         ----------
         point : (float, float)
             The point (x, y) to check.
-        transform : `matplotlib.transforms.Transform`, optional
+        transform : `~matplotlib.transforms.Transform`, optional
             If not ``None``, *point* will be compared to ``self`` transformed
             by *transform*; i.e. for a correct check, *transform* should
             transform the path into the coordinate system of *point*.
         radius : float, default: 0
-            Add an additional margin on the path in coordinates of *point*.
+            Additional margin on the path in coordinates of *point*.
             The path is extended tangentially by *radius/2*; i.e. if you would
             draw the path with a linewidth of *radius*, all points on the line
             would still be considered to be contained in the area. Conversely,
@@ -546,19 +554,19 @@ class Path:
         Return whether the area enclosed by the path contains the given points.
 
         The path is always treated as closed; i.e. if the last code is not
-        CLOSEPOLY an implicit segment connecting the last vertex to the first
+        `CLOSEPOLY` an implicit segment connecting the last vertex to the first
         vertex is assumed.
 
         Parameters
         ----------
         points : (N, 2) array
             The points to check. Columns contain x and y values.
-        transform : `matplotlib.transforms.Transform`, optional
+        transform : `~matplotlib.transforms.Transform`, optional
             If not ``None``, *points* will be compared to ``self`` transformed
             by *transform*; i.e. for a correct check, *transform* should
             transform the path into the coordinate system of *points*.
         radius : float, default: 0
-            Add an additional margin on the path in coordinates of *points*.
+            Additional margin on the path in coordinates of *points*.
             The path is extended tangentially by *radius/2*; i.e. if you would
             draw the path with a linewidth of *radius*, all points on the line
             would still be considered to be contained in the area. Conversely,
@@ -602,7 +610,7 @@ class Path:
 
         Parameters
         ----------
-        transform : matplotlib.transforms.Transform, optional
+        transform : `~matplotlib.transforms.Transform`, optional
             Transform to apply to path before computing extents, if any.
         **kwargs
             Forwarded to `.iter_bezier`.
@@ -660,9 +668,9 @@ class Path:
 
     def interpolated(self, steps):
         """
-        Return a new path resampled to length N x steps.
+        Return a new path resampled to length N x *steps*.
 
-        Codes other than LINETO are not handled correctly.
+        Codes other than `LINETO` are not handled correctly.
         """
         if steps == 1:
             return self
@@ -680,15 +688,18 @@ class Path:
     def to_polygons(self, transform=None, width=0, height=0, closed_only=True):
         """
         Convert this path to a list of polygons or polylines.  Each
-        polygon/polyline is an Nx2 array of vertices.  In other words,
-        each polygon has no ``MOVETO`` instructions or curves.  This
+        polygon/polyline is an (N, 2) array of vertices.  In other words,
+        each polygon has no `MOVETO` instructions or curves.  This
         is useful for displaying in backends that do not support
-        compound paths or Bezier curves.
+        compound paths or Bézier curves.
 
         If *width* and *height* are both non-zero then the lines will
         be simplified so that vertices outside of (0, 0), (width,
         height) will be clipped.
 
+        The resulting polygons will be simplified if the
+        :attr:`Path.should_simplify` attribute of the path is `True`.
+
         If *closed_only* is `True` (default), only closed polygons,
         with the last point being the same as the first point, will be
         returned.  Any unclosed polylines in the path will be
@@ -822,7 +833,7 @@ class Path:
 
         Notes
         -----
-        The circle is approximated using 8 cubic Bezier curves, as described in
+        The circle is approximated using 8 cubic Bézier curves, as described in
 
           Lancaster, Don.  `Approximating a Circle or an Ellipse Using Four
           Bezier Cubic Splines <https://www.tinaja.com/glib/ellipse4.pdf>`_.
@@ -920,8 +931,8 @@ class Path:
     @classmethod
     def arc(cls, theta1, theta2, n=None, is_wedge=False):
         """
-        Return the unit circle arc from angles *theta1* to *theta2* (in
-        degrees).
+        Return a `Path` for the unit circle arc from angles *theta1* to
+        *theta2* (in degrees).
 
         *theta2* is unwrapped to produce the shortest arc within 360 degrees.
         That is, if *theta2* > *theta1* + 360, the arc will be from *theta1* to
@@ -933,7 +944,7 @@ class Path:
 
            Masionobe, L.  2003.  `Drawing an elliptical arc using
            polylines, quadratic or cubic Bezier curves
-           <http://www.spaceroots.org/documents/ellipse/index.html>`_.
+           <https://web.archive.org/web/20190318044212/http://www.spaceroots.org/documents/ellipse/index.html>`_.
         """
         halfpi = np.pi * 0.5
 
@@ -999,8 +1010,8 @@ class Path:
     @classmethod
     def wedge(cls, theta1, theta2, n=None):
         """
-        Return the unit circle wedge from angles *theta1* to *theta2* (in
-        degrees).
+        Return a `Path` for the unit circle wedge from angles *theta1* to
+        *theta2* (in degrees).
 
         *theta2* is unwrapped to produce the shortest wedge within 360 degrees.
         That is, if *theta2* > *theta1* + 360, the wedge will be from *theta1*
@@ -1018,7 +1029,7 @@ class Path:
     @lru_cache(8)
     def hatch(hatchpattern, density=6):
         """
-        Given a hatch specifier, *hatchpattern*, generates a Path that
+        Given a hatch specifier, *hatchpattern*, generates a `Path` that
         can be used in a repeated hatching pattern.  *density* is the
         number of lines per unit square.
         """
@@ -1036,7 +1047,6 @@ class Path:
         If *inside* is `True`, clip to the inside of the box, otherwise
         to the outside of the box.
         """
-        # Use make_compound_path_from_polys
         verts = _path.clip_path_to_rect(self, bbox, inside)
         paths = [Path(poly) for poly in verts]
         return self.make_compound_path(*paths)
@@ -1045,32 +1055,41 @@ class Path:
 def get_path_collection_extents(
         master_transform, paths, transforms, offsets, offset_transform):
     r"""
-    Given a sequence of `Path`\s, `~.Transform`\s objects, and offsets, as
-    found in a `~.PathCollection`, returns the bounding box that encapsulates
-    all of them.
+    Get bounding box of a `.PathCollection`\s internal objects.
+
+    That is, given a sequence of `Path`\s, `.Transform`\s objects, and offsets, as found
+    in a `.PathCollection`, return the bounding box that encapsulates all of them.
 
     Parameters
     ----------
-    master_transform : `~.Transform`
+    master_transform : `~matplotlib.transforms.Transform`
         Global transformation applied to all paths.
     paths : list of `Path`
-    transforms : list of `~.Affine2D`
+    transforms : list of `~matplotlib.transforms.Affine2DBase`
+        If non-empty, this overrides *master_transform*.
     offsets : (N, 2) array-like
-    offset_transform : `~.Affine2D`
+    offset_transform : `~matplotlib.transforms.Affine2DBase`
         Transform applied to the offsets before offsetting the path.
 
     Notes
     -----
-    The way that *paths*, *transforms* and *offsets* are combined
-    follows the same method as for collections:  Each is iterated over
-    independently, so if you have 3 paths, 2 transforms and 1 offset,
-    their combinations are as follows:
+    The way that *paths*, *transforms* and *offsets* are combined follows the same
+    method as for collections: each is iterated over independently, so if you have 3
+    paths (A, B, C), 2 transforms (α, β) and 1 offset (O), their combinations are as
+    follows:
 
-        (A, A, A), (B, B, A), (C, A, A)
+    - (A, α, O)
+    - (B, β, O)
+    - (C, α, O)
     """
     from .transforms import Bbox
     if len(paths) == 0:
         raise ValueError("No paths provided")
+    if len(offsets) == 0:
+        _api.warn_deprecated(
+            "3.8", message="Calling get_path_collection_extents() with an"
+            " empty offsets list is deprecated since %(since)s. Support will"
+            " be removed %(removal)s.")
     extents, minpos = _path.get_path_collection_extents(
         master_transform, paths, np.atleast_3d(transforms),
         offsets, offset_transform)
