@@ -1,10 +1,10 @@
 from io import BytesIO, StringIO
+import gc
 import multiprocessing
 import os
 from pathlib import Path
 from PIL import Image
 import shutil
-import subprocess
 import sys
 import warnings
 
@@ -14,9 +14,10 @@ import pytest
 from matplotlib.font_manager import (
     findfont, findSystemFonts, FontEntry, FontProperties, fontManager,
     json_dump, json_load, get_font, is_opentype_cff_font,
-    MSUserFontDirectories, _get_fontconfig_fonts, ft2font,
-    ttfFontProperty, cbook)
-from matplotlib import pyplot as plt, rc_context
+    MSUserFontDirectories, _get_fontconfig_fonts, ttfFontProperty)
+from matplotlib import cbook, ft2font, pyplot as plt, rc_context, figure as mfigure
+from matplotlib.testing import subprocess_run_helper, subprocess_run_for_testing
+
 
 has_fclist = shutil.which('fc-list') is not None
 
@@ -25,11 +26,11 @@ def test_font_priority():
     with rc_context(rc={
             'font.sans-serif':
             ['cmmi10', 'Bitstream Vera Sans']}):
-        font = findfont(FontProperties(family=["sans-serif"]))
-    assert Path(font).name == 'cmmi10.ttf'
+        fontfile = findfont(FontProperties(family=["sans-serif"]))
+    assert Path(fontfile).name == 'cmmi10.ttf'
 
     # Smoketest get_charmap, which isn't used internally anymore
-    font = get_font(font)
+    font = get_font(fontfile)
     cmap = font.get_charmap()
     assert len(cmap) == 131
     assert cmap[8729] == 30
@@ -46,12 +47,11 @@ def test_score_weight():
             fontManager.score_weight(400, 400))
 
 
-def test_json_serialization(tmpdir):
+def test_json_serialization(tmp_path):
     # Can't open a NamedTemporaryFile twice on Windows, so use a temporary
     # directory instead.
-    path = Path(tmpdir, "fontlist.json")
-    json_dump(fontManager, path)
-    copy = json_load(path)
+    json_dump(fontManager, tmp_path / "fontlist.json")
+    copy = json_load(tmp_path / "fontlist.json")
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', 'findfont: Font family.*not found')
         for prop in ({'family': 'STIXGeneral'},
@@ -133,8 +133,7 @@ def test_find_noto():
         fig.savefig(BytesIO(), format=fmt)
 
 
-def test_find_invalid(tmpdir):
-    tmp_path = Path(tmpdir)
+def test_find_invalid(tmp_path):
 
     with pytest.raises(FileNotFoundError):
         get_font(tmp_path / 'non-existent-font-name.ttf')
@@ -148,7 +147,7 @@ def test_find_invalid(tmpdir):
     # Not really public, but get_font doesn't expose non-filename constructor.
     from matplotlib.ft2font import FT2Font
     with pytest.raises(TypeError, match='font file or a binary-mode file'):
-        FT2Font(StringIO())
+        FT2Font(StringIO())  # type: ignore[arg-type]
 
 
 @pytest.mark.skipif(sys.platform != 'linux' or not has_fclist,
@@ -277,15 +276,30 @@ def _test_threading():
 
 def test_fontcache_thread_safe():
     pytest.importorskip('threading')
-    import inspect
 
-    proc = subprocess.run(
-        [sys.executable, "-c",
-         inspect.getsource(_test_threading) + '\n_test_threading()']
+    subprocess_run_helper(_test_threading, timeout=10)
+
+
+def test_lockfilefailure(tmp_path):
+    # The logic here:
+    # 1. get a temp directory from pytest
+    # 2. import matplotlib which makes sure it exists
+    # 3. get the cache dir (where we check it is writable)
+    # 4. make it not writable
+    # 5. try to write into it via font manager
+    proc = subprocess_run_for_testing(
+        [
+            sys.executable,
+            "-c",
+            "import matplotlib;"
+            "import os;"
+            "p = matplotlib.get_cachedir();"
+            "os.chmod(p, 0o555);"
+            "import matplotlib.font_manager;"
+        ],
+        env={**os.environ, 'MPLCONFIGDIR': str(tmp_path)},
+        check=True
     )
-    if proc.returncode:
-        pytest.fail("The subprocess returned with non-zero exit status "
-                    f"{proc.returncode}.")
 
 
 def test_fontentry_dataclass():
@@ -317,10 +331,32 @@ def test_get_font_names():
             font = ft2font.FT2Font(path)
             prop = ttfFontProperty(font)
             ttf_fonts.append(prop.name)
-        except:
+        except Exception:
             pass
     available_fonts = sorted(list(set(ttf_fonts)))
     mpl_font_names = sorted(fontManager.get_font_names())
     assert set(available_fonts) == set(mpl_font_names)
     assert len(available_fonts) == len(mpl_font_names)
     assert available_fonts == mpl_font_names
+
+
+def test_donot_cache_tracebacks():
+
+    class SomeObject:
+        pass
+
+    def inner():
+        x = SomeObject()
+        fig = mfigure.Figure()
+        ax = fig.subplots()
+        fig.text(.5, .5, 'aardvark', family='doesnotexist')
+        with BytesIO() as out:
+            with warnings.catch_warnings():
+                warnings.filterwarnings('ignore')
+                fig.savefig(out, format='raw')
+
+    inner()
+
+    for obj in gc.get_objects():
+        if isinstance(obj, SomeObject):
+            pytest.fail("object from inner stack still alive")
