@@ -7,7 +7,6 @@
 #define PY_SSIZE_T_CLEAN
 
 #include <Python.h>
-#include <stdint.h>
 
 #ifdef _POSIX_C_SOURCE
 #    undef _POSIX_C_SOURCE
@@ -49,8 +48,6 @@ enum {
     CLOSEPOLY = 0x4f
 };
 
-const size_t NUM_VERTICES[] = { 1, 1, 1, 2, 3, 1 };
-
 inline int prepare_and_add_type(PyTypeObject *type, PyObject *module)
 {
     if (PyType_Ready(type)) {
@@ -74,10 +71,10 @@ inline int prepare_and_add_type(PyTypeObject *type, PyObject *module)
 template<typename T>
 inline bool check_trailing_shape(T array, char const* name, long d1)
 {
-    if (array.dim(1) != d1) {
+    if (array.shape(1) != d1) {
         PyErr_Format(PyExc_ValueError,
                      "%s must have shape (N, %ld), got (%ld, %ld)",
-                     name, d1, array.dim(0), array.dim(1));
+                     name, d1, array.shape(0), array.shape(1));
         return false;
     }
     return true;
@@ -86,10 +83,10 @@ inline bool check_trailing_shape(T array, char const* name, long d1)
 template<typename T>
 inline bool check_trailing_shape(T array, char const* name, long d1, long d2)
 {
-    if (array.dim(1) != d1 || array.dim(2) != d2) {
+    if (array.shape(1) != d1 || array.shape(2) != d2) {
         PyErr_Format(PyExc_ValueError,
                      "%s must have shape (N, %ld, %ld), got (%ld, %ld, %ld)",
-                     name, d1, d2, array.dim(0), array.dim(1), array.dim(2));
+                     name, d1, d2, array.shape(0), array.shape(1), array.shape(2));
         return false;
     }
     return true;
